--[[*************************************************************
File: Core.lua          AddOn: SLFriendList
Rev Date: 12/14/2007    Author: Taffu 
***************************************************************]]

local L = AceLibrary("AceLocale-2.2"):new("SLFriendList")
local SML = AceLibrary("LibSharedMedia-2.0")

SLFriendList = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "SLModule-1.0")
SLFriendList:RegisterDB("SLFListVar")

local db, cdb, text
local self = SLFriendList

function SLFriendList:OnInitialize()
    self:RegisterDefaults("profile", self.defaults)
    self:RegisterDefaults("char", {} )
    self:RegisterChatCommand("/slfriendlist", self.opts)
    db = self.db.profile
    cdb = self.db.char
    -- Register this module
    if not IsAddOnLoaded("SLCore") then error("SLFriendList requires SLCore") else
        self:RegisterSLModule(SLFriendList, "SLFriendList")
    end
    -- Create frame
    self:CreateFrame()
end

function SLFriendList:OnEnable()
    -- Event Updates
    self:RegisterEvent("FRIENDLIST_UPDATE", "UpdateMod")
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateMod")
    -- Update the options on enable
    self:BuildOptions()
    self.frame:Show()
end

function SLFriendList:OnDisable()
    -- Hide frame and update vars enabled
    self.frame:Hide()
end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
-- These two functions are required, and must be present in all SL modules
------------------------------------------------------------------------------------
function SLFriendList:Update() -- Calls mixin's as well as text updates
    -- Call both updates to avoid delay in display when updating settings
    self:UpdateFrame(self) -- Library mixin to update this modules frame
    self:UpdateMod() -- Call text update function
end

function SLFriendList:UpdateMod()
    local text1, text2
    local color1, color2
    
    if db.isMasterColors then
        color1 = self:GetColorValue(SLCore.db.profile.masterColor1)
        color2 = self:GetColorValue(SLCore.db.profile.masterColor2)
    else 
        color1 = self:GetColorValue(db.color1)
        color2 = self:GetColorValue(db.color2)
    end
    
    -- We need to clear the table data to give a fresh update
    cdb.FriendList = nil
    -- Ok, we need the total number of friends on the friend list first
    local friendtotal = GetNumFriends()
    local friendonline = 0
    -- Now lets get our count of how many friends are online
    for i = 0, friendtotal do
        local name, lvl, class, area, online, status = GetFriendInfo(i)
        if online then
            friendonline = friendonline + 1
            -- If they're online, we'll add them to our friend table
            if not cdb.FriendList then cdb.FriendList = {} end
            local classcolor = self:GetColorValue(string.format("%s", class))
            local coloredclass = string.format("%s%s|r", classcolor, class)
            table.insert(cdb.FriendList, { name, lvl, coloredclass, area, status })
        end
    end
    
    text2 = db.basetext
    text1 = friendonline
    
    -- Now setup our display - All SL modules require this formatted string
    text = string.format("%s%s|r %s%s|r", color2, text2, color1, text1)
    -- Now update module text with our new text
    self:UpdateText(self, text) -- Library mixin to update this modules text
    
    -- Let's setup our tooltip
    if db.enableTT and friendonline > 0 and SLCore.db.profile.isLocked then
        self.frame:EnableMouse(true)
        self.frame:SetScript("OnEnter", function(this)
            if SLCore.db.profile.isLocked and db.enableTT then
                if db.modifyTT then
                    if IsShiftKeyDown() then
                        GameTooltip:SetOwner(this, "ANCHOR_CURSOR")
                        GameTooltip:AddLine(string.format("%s%s|r", color2, "Friends Online"))
                        GameTooltip:AddLine("-------------------------", 1, 1, 1)
                        for _, val in ipairs(cdb.FriendList) do
                            local ptext = string.format("%s%s%s|r: %s %s", val[5], color2, val[1], val[2], val[3])
                            local dtext = string.format("%sLocation:|r %s", color2, val[4])
                            GameTooltip:AddDoubleLine(ptext, dtext, 1, 1, 1, 1, 1, 1)
                        end
                        GameTooltip:SetScale(db.scaleTT)
                        GameTooltip:Show()
                    end
                else
                    GameTooltip:SetOwner(this, "ANCHOR_CURSOR")
                    GameTooltip:AddLine(string.format("%s%s|r", color2, "Friends Online"))
                    GameTooltip:AddLine("-------------------------", 1, 1, 1)
                    for _, val in ipairs(cdb.FriendList) do
                        local ptext = string.format("%s%s%s|r: %s %s", val[5], color2, val[1], val[2], val[3])
                        local dtext = string.format("%sLocation:|r %s", color2, val[4])
                        GameTooltip:AddDoubleLine(ptext, dtext, 1, 1, 1, 1, 1, 1)
                    end
                    GameTooltip:SetScale(db.scaleTT)
                    GameTooltip:Show()
                end
            end
        end)
        self.frame:SetScript("OnLeave", function()
            if GameTooltip:IsShown() then
                GameTooltip:Hide()
            end
        end)
    else
        self.frame:SetScript("OnEnter", nil)
        self.frame:SetScript("OnLeave", nil)
    end
end
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function SLFriendList:CreateFrame()
    if self.frame then return else
        -- Frame & Text
        self.frame = CreateFrame("Frame", nil, UIParent)
        self.text = self.frame:CreateFontString(nil, "OVERLAY")
        -- Stat Seperator (Coming Soon)
        -- self.sep = self.frame:CreateFontString(nil, "OVERLAY")
        -- Now update our module frame properties
        self:UpdateFrame(self)
        -- Name this frame as an anchor usable in-game (add to library later)
        SLFriendListAnchor = self.frame
    end
end

function SLFriendList:MoveFrame()
    local mdb = SLCore.db.profile
    self.frame:SetPoint(db.justify, db.pos.anchor, db.pos.anchorFrom, db.pos.x, db.pos.y)
    if not mdb.isLocked then self.frame:StartMoving() else end
end

function SLFriendList:StopFrame()
    local mdb = SLCore.db.profile
    self.frame:StopMovingOrSizing()
    local aP, _, aF, x, y = self.frame:GetPoint()
    local anchor = self.frame:GetParent():GetName()
    local offX, offY = self:TranslateCoords(self, db.justify, aF, x, y)
    db.pos.anchorPoint, db.pos.anchor, db.pos.anchorFrom, db.pos.x, db.pos.y = db.justify, anchor, aF, floor(offX), floor(offY)
end

function SLFriendList:Print(msg)
    if msg then
        DEFAULT_CHAT_FRAME:AddMessage(msg)
    end
end

-------------------------------------------
--  Options & Defaults
-------------------------------------------

SLFriendList.defaults = {
    basetext = "Friends:",
    enableTT = true,
    modifyTT = false,
    scaleTT = 1,
    color1 = "White",
    color2 = "White",
    fontFace = "Arial Narrow",
    fontSize = 16,
    isMasterFont = true,
    isMasterFontSize = true,
    isMasterColors = true,
    pos = { anchorPoint = "RIGHT", anchor = "UIParent", anchorFrom = "CENTER", x = 250, y = 25 },
    justify = "RIGHT",
    frameStrata = "BACKGROUND",
    anchToSL = false,
}

SLFriendList.opts = {
    type = "group",
    args = {
        basetext = {
            type = "text",
            name = L["Base Text"],
            desc = L["Base Text Desc"],
            usage = L["<Base Text>"],
            get = function() return db.basetext end,
            set = function(v) db.basetext = v self:Update() end,
            order = 10,
        },
        enableTT = {
            type = "toggle",
            name = L["Enable Tooltip"],
            desc = L["Enable Tooltip Desc"],
            get = function() return db.enableTT end,
            set = function() db.enableTT = not db.enableTT self:Update() end,
            order = 15,
        },
        modifyTT = {
            type = "toggle",
            name = L["Tooltip Modifier"],
            desc = L["Tooltip Modifier Desc"],
            disabled = function() if db.enableTT then return false else return true end end,
            get = function() return db.modifyTT end,
            set = function() db.modifyTT = not db.modifyTT self:Update() end,
            order = 20,
        },
        inheritColors = {
            type = "toggle",
            name = L["Inherit Master Colors"],
            desc = L["Inherit Master Colors Desc"],
            get = function() return db.isMasterColors end,
            set = function() db.isMasterColors = not db.isMasterColors self:Update() end,
            order = 40,
        },
        textColor1 = {
            type = "text",
            name = L["First Color"],
            desc = L["First Color Desc"],
            validate = SLCore.Colorlist,
            disabled = function() if db.isMasterColors then return true else return false end end,
            get = function() return db.color1 end,
            set = function(v) db.color1 = v self:Update() end,
            order = 50,
        },
        textColor2 = {
            type = "text",
            name = L["Second Color"],
            desc = L["Second Color Desc"],
            validate = SLCore.Colorlist,
            disabled = function() if db.isMasterColors then return true else return false end end,
            get = function() return db.color2 end,
            set = function(v) db.color2 = v self:Update() end,
            order = 60,
        },
        inheritFont = {
            type = "toggle",
            name = L["Inherit Master Font"],
            desc = L["Inherit Master Font Desc"],
            get = function() return db.isMasterFont end,
            set = function() db.isMasterFont = not db.isMasterFont self:Update() end,
            order = 70,
        },
        inheritFontSize = {
            type = "toggle",
            name = L["Inherit Master Font Size"],
            desc = L["Inherit Master Font Size Desc"],
            get = function() return db.isMasterFontSize end,
            set = function() db.isMasterFontSize = not db.isMasterFontSize self:Update() end,
            order = 80,
        },
        font = {
            type = "text",
            name = L["Font Face"],
            desc = L["Font Face Desc"],
            validate = SML:List("font"),
            disabled = function() if db.isMasterFont then return true else return false end end,
            get = function() return db.fontFace end,
            set = function(v) db.fontFace = v  self:Update() end,
            order = 90,
        },
        fontSize = {
            type = "range",
            name = L["Font Size"],
            desc = L["Font Size Desc"],
            min = 4,
            max = 36,
            step = 1,
            disabled = function() if db.isMasterFontSize then return true else return false end end,
            get = function() return db.fontSize end,
            set = function(v) db.fontSize = v self:Update() end,
            order = 100,
        },
        justify = {
            type = "text",
            name = L["Text Justify"],
            desc = L["Text Justify Desc"],
            validate = SLCore.JustifyTable,
            get = function() return db.justify end,
            set = function(v) db.justify = v self:Update() end,
            order = 110,
        },
        offsetX = {
            type = "range",
            name = L["X Offset"],
            desc = L["X Offset Desc"],
            min = -1000,
            max = 1000,
            step = 1,
            get = function() return db.pos.x end,
            set = function(v) db.pos.x = v self:Update() end,
            order = 120,
        },
        offsetY = {
            type = "range",
            name = L["Y Offset"],
            desc = L["Y Offset Desc"],
            min = -1000,
            max = 1000,
            step = 1,
            get = function() return db.pos.y end,
            set = function(v) db.pos.y = v self:Update() end,
            order = 125,
        },
        frameStrata = {
            type = "text",
            name = L["Frame Strata"],
            desc = L["Frame Strata Desc"],
            validate = SLCore.StrataTable,
            get = function() return db.frameStrata end,
            set = function(v) db.frameStrata = v self:Update() end,
            order = 135,
        },
        parentFrame = {
            type = "text",
            name = L["Parent Frame"],
            desc = L["Parent Frame Desc"],
            usage = L["<Parent Frame>"],
            get = function() return db.pos.anchor end,
            set = function(v) db.pos.anchor = v self:Update() end,
            order = 140,
        },
    },
}